<?php

namespace App\Factory;

use App\Exception;

class PriceFactory {
    
    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function clientBranchPricingTable($CDCLIENTE, $CDFILIAL) {
        $sql = "
    		SELECT CDTABEPREC, CDCFILTABPRE AS CDFILTABPREC
    		  FROM CLIENFILIAL
    		 WHERE CDCLIENTE = :CDCLIENTE
    		   AND CDFILIAL  = :CDFILIAL
	    ";
        
        $params = [
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDFILIAL' => $CDFILIAL
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result[0]['CDTABEPREC']) ? $result[0] : []);
    }

    public function clientPricingTable($CDCLIENTE) {
        $sql = "
    		SELECT CDTABEPREC, CDFILTABPREC AS CDFILTABPREC
    		  FROM CLIENTE
    		 WHERE CDCLIENTE = :CDCLIENTE
    	";
        
        $params = [
            ':CDCLIENTE' => $CDCLIENTE
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result[0]['CDTABEPREC']) ? $result[0] : []);
    }

    public function storePricingTable($CDFILIAL, $CDLOJA) {
        $sql = "
    		SELECT CDTABEPREC, NULL AS CDFILTABPREC
    		  FROM LOJA
    		 WHERE CDFILIAL = :CDFILIAL
    		   AND CDLOJA   = :CDLOJA
    	";
        
        $params = [
            ':CDFILIAL' => $CDFILIAL,
            ':CDLOJA' => $CDLOJA
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result[0]['CDTABEPREC']) ? $result[0] : []);
    }

    public function paravendPricingTable($CDFILIAL){
        $sql = "
    		SELECT NVL(CDTABEPREC, CDTABEPRECDLV) AS CDTABEPREC, NULL AS CDFILTABPREC
    		  FROM PARAVEND
    		 WHERE CDFILIAL = :CDFILIAL
    	";
    	
        $params = [
            ':CDFILIAL' => $CDFILIAL
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result[0]['CDTABEPREC']) ? $result[0] : []);
    }
    
    public function generalPricingTable($CDFILIAL){
        $sql = "
    		SELECT T.CDTABEPREC, NULL AS CDFILTABPREC
              FROM TABEVENDGER T, FILITABGER FT
                WHERE (FT.CDFILIAL = :CDFILIAL)
                AND (T.CDTABEPREC = FT.CDTABEPREC)
                AND (SYSDATE BETWEEN DTINIVGPREC AND DTFINVGPREC)
    	";
    	
        $params = [
            ':CDFILIAL' => $CDFILIAL
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result[0]['CDTABEPREC']) ? $result[0]['CDTABEPREC'] : null);
    }
    
}