<?php
namespace App\Service;

use App\Exception as Exception;

class PriceService {
    
    private $priceFactory;
    private $menuFactory;

    public function __construct(\App\Factory\MenuFactory $menuFactory,
                                \App\Factory\PriceFactory $priceFactory
                                ) {
        $this->priceFactory = $priceFactory;
        $this->menuFactory = $menuFactory;
    }
    
    public function getPriceTable($CDCLIENTE, $CDFILIAL, $CDLOJA){
            $pricingTable = $this->priceFactory->clientBranchPricingTable($CDCLIENTE, $CDFILIAL);
            
            if(empty($pricingTable)) {
                $pricingTable = $this->priceFactory->clientPricingTable($CDCLIENTE);
                
                if (empty($pricingTable)) {
                    $pricingTable = $this->priceFactory->storePricingTable($CDFILIAL, $CDLOJA);
                    
                    if (empty($pricingTable)) {
                        $pricingTable = $this->priceFactory->paravendPricingTable($CDFILIAL);
                    }
                }
            }
            
            $pricingTable['TABEVENDGER'] = $this->priceFactory->generalPricingTable($CDFILIAL);
    
            
            if(!empty($pricingTable)) {
                if ($pricingTable['CDFILTABPREC'] === null) {
                    $pricingTable['CDFILTABPREC'] = $CDFILIAL;
                }
            } else {
                throw new \UnexpectedValueException('Não há tabela de preços cadastrada.');
            }

            return $pricingTable;
    }
    
    public function getGeneralPricingTable($CDFILIAL){
        
        $pricingTable = $this->priceFactory->getGeneralPricingTable($CDFILIAL);
            
        if(empty($pricingTable)) {
            throw new \Exception('Não há tabela de preços cadastrada.');
        }

        return $pricingTable['CDTABEPREC'];
    }
    
    public function checkEffective($CDFILIAL, $priceTable) {
        try {
            $salesTable = $this->priceFactory->getSalesTable($CDFILIAL, $priceTable);
            
            if (!empty($salesTable)) 
                return $salesTable['DTINIVGPREC'];
            else 
                throw new \UnexpectedValueException('Tabela de preços está inativa.');

        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function getGeneralPricingTableValidity($priceTable) {
        $salesTable = $this->priceFactory->getGeneralPricingTableValidity($priceTable);
        
        if (empty($salesTable)) 
            throw new \Exception('Validade da tabela de preços expirada ou inexistente.');

        return $salesTable['DTINIVGPREC'];
    }
    
    public function isHoliday($CDFILIAL, $currentDay) {
        try {
            $check = $this->priceFactory->holidayCheck($CDFILIAL, $currentDay);
            
            if (!empty($check)) 
                return true;
            
            return false;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function getProductDiscountType($CDFILIAL, $CDTIPOCONS) {
        return $this->priceFactory->getProductDiscountType($CDFILIAL, $CDTIPOCONS);
    }
    
    public function getPricesOnHoliday($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $currentDay) {
        return $this->priceFactory->getPricesOnHoliday($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $currentDay);
    }
    
    public function getPricesByDay($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $currentDay, $CDTIPOCONSPD) {
        return $this->priceFactory->getPricesByDay($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $currentDay, $CDTIPOCONSPD);
    }
    
    public function getPrices($CDFILIAL, $CDTABEPREC, $DTINIVGPREC) {
        try {
            $prices = $this->priceFactory->getPrices($CDFILIAL, $CDTABEPREC, $DTINIVGPREC);
            
            if (!empty($prices)) {
                $indexedPrices = array();

                foreach ($prices as $priceItem){
                    $index = $priceItem["CDPRODUTO"];
                    unset($priceItem["CDPRODUTO"]);
                    $indexedPrices[$index] = $priceItem;
                }

                return $indexedPrices;
            } else {
                throw new \UnexpectedValueException("Não existem precos cadastrados.");
            }
        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function getGeneralPrices($CDTABEPREC, $DTINIVGPREC) {
        $prices = $this->priceFactory->getGeneralPrices($CDTABEPREC, $DTINIVGPREC);
        
        if (empty($prices)) {
            throw new \Exception("Não foi possível buscar precos associados a tabela de preco geral $CDTABEPREC.");
        }
        
        $indexedPrices = array();

        foreach ($prices as $priceItem){
            $index = $priceItem["CDPRODUTO"];
            $indexedPrices[$index] = $priceItem;
        }

        return $indexedPrices;
    }
    
    public function getDiffPrices($CDTABEPREC, $NRDIASEMANPR) {
        $prices = $this->priceFactory->getDiffPrices($CDTABEPREC, $NRDIASEMANPR);
        
        $indexedPrices = array();

        foreach ($prices as $priceItem){
            $index = $priceItem["CDPRODUTO"];
            unset($priceItem["CDPRODUTO"]);
            $indexedPrices[$index] = $priceItem;
        }

        return $indexedPrices;
    }
    
    public function checkPrice($CDFILIAL, $pricingTable, $productCode, $effectiveDate) {
        try {

            $itemPrice = $this->priceFactory->getItemPrice($CDFILIAL, $pricingTable, $effectiveDate, $productCode);

            if (empty($itemPrice)) {
                throw new \UnexpectedValueException('Preço não cadastrado.');
            } else if ($itemPrice['FULLPRICE']+$itemPrice['PRECOCLIE'] === 0) {
                throw new \UnexpectedValueException('Produto sem preço cadastrado.');
            } else {
                return $itemPrice;
            }

        } catch (\UnexpectedValueException $e) {
            throw new \Exception($e->getMessage());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
}